package eu.eudat.data.dao.entities;

import eu.eudat.data.dao.DatabaseAccess;
import eu.eudat.data.dao.criteria.DataManagementPlanCriteria;
import eu.eudat.data.dao.criteria.DatasetWizardUserDmpCriteria;
import eu.eudat.data.dao.databaselayer.service.DatabaseService;
import eu.eudat.data.entities.DMP;
import eu.eudat.data.entities.UserInfo;
import eu.eudat.queryable.QueryableList;
import eu.eudat.queryable.types.FieldSelectionType;
import eu.eudat.queryable.types.SelectionField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

@Component("dMPDao")
public class DMPDaoImpl extends DatabaseAccess<DMP> implements DMPDao {


    @Autowired
    public DMPDaoImpl(DatabaseService<DMP> databaseService) {
        super(databaseService);
    }

    @Override
    public QueryableList<DMP> getWithCriteria(DataManagementPlanCriteria criteria) {
        QueryableList<DMP> query = getDatabaseService().getQueryable(DMP.getHints(), DMP.class);
        if (criteria.getLike() != null && !criteria.getLike().isEmpty())
            query.where((builder, root) -> builder.like(root.get("label"), "%" + criteria.getLike() + "%"));
        if (criteria.getPeriodEnd() != null)
            query.where((builder, root) -> builder.lessThan(root.get("created"), criteria.getPeriodEnd()));
        if (criteria.getPeriodStart() != null)
            query.where((builder, root) -> builder.greaterThan(root.get("created"), criteria.getPeriodStart()));
        if (criteria.getProjects() != null && !criteria.getProjects().isEmpty())
            query.where(((builder, root) -> root.get("project").in(criteria.getProjects())));
        if (!criteria.getAllVersions())
            query.initSubQuery(String.class).where((builder, root) -> builder.equal(root.get("version"), query.<String>subQueryMax((builder1, externalRoot, nestedRoot) -> builder1.equal(externalRoot.get("groupId"), nestedRoot.get("groupId")), Arrays.asList(new SelectionField(FieldSelectionType.FIELD, "version")), String.class)));
        if (criteria.getGroupIds() != null && !criteria.getGroupIds().isEmpty())
            query.where((builder, root) -> root.get("groupId").in(criteria.getGroupIds()));
        query.where((builder, root) -> builder.notEqual(root.get("status"), DMP.DMPStatus.DELETED.getValue()));
        return query;
    }

    public QueryableList<DMP> getAuthenticated(QueryableList<DMP> query, UserInfo principal) {
        query.where((builder, root) -> builder.or(builder.equal(root.get("creator"), principal), builder.isMember(principal, root.get("users"))));
        return query;
    }

    @Override
    public DMP createOrUpdate(DMP item) {
        return this.getDatabaseService().createOrUpdate(item, DMP.class);
    }

    @Override
    public DMP find(UUID id) {
        return getDatabaseService().getQueryable(DMP.class).where((builder, root) -> builder.equal((root.get("id")), id)).getSingle();
    }

    @Override
    public QueryableList<DMP> getUserDmps(DatasetWizardUserDmpCriteria datasetWizardUserDmpCriteria, UserInfo userInfo) {
        QueryableList<DMP> query = getDatabaseService().getQueryable(DMP.class).where((builder, root) -> builder.or(builder.equal(root.get("creator"), userInfo), builder.isMember(userInfo, root.get("users"))));
        if (datasetWizardUserDmpCriteria.getLike() != null && !datasetWizardUserDmpCriteria.getLike().isEmpty()) {
            query.where((builder, root) -> builder.like(root.get("label"), "%" + datasetWizardUserDmpCriteria.getLike() + "%"));
        }
        return query;
    }

    @Override
    public void delete(DMP item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<DMP> asQueryable() {
        return this.getDatabaseService().getQueryable(DMP.class);
    }

    @Override
    public CompletableFuture<DMP> createOrUpdateAsync(DMP item) {
        return CompletableFuture.supplyAsync(() -> this.createOrUpdate(item));
    }

    @Override
    public DMP find(UUID id, String hint) {
        throw new UnsupportedOperationException();
    }
}
