package eu.eudat.data.dao.entities;

import eu.eudat.data.dao.DatabaseAccess;
import eu.eudat.data.dao.criteria.DatasetProfileCriteria;
import eu.eudat.data.dao.databaselayer.service.DatabaseService;
import eu.eudat.data.entities.DatasetProfile;
import eu.eudat.queryable.QueryableList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.UUID;
import java.util.concurrent.CompletableFuture;

@Component("datasetProfileDao")
public class DatasetProfileDaoImpl extends DatabaseAccess<DatasetProfile> implements DatasetProfileDao {

    @Autowired
    public DatasetProfileDaoImpl(DatabaseService<DatasetProfile> databaseService) {
        super(databaseService);
    }

    @Override
    public QueryableList<DatasetProfile> getWithCriteria(DatasetProfileCriteria criteria) {
        QueryableList<DatasetProfile> query = getDatabaseService().getQueryable(DatasetProfile.class);
        if (criteria.getLike() != null && !criteria.getLike().isEmpty())
            query.where((builder, root) -> builder.like(root.get("label"), "%" + criteria.getLike() + "%"));
        return query;
    }

    @Override
    public DatasetProfile createOrUpdate(DatasetProfile item) {
        return this.getDatabaseService().createOrUpdate(item, DatasetProfile.class);
    }

    @Override
    public DatasetProfile find(UUID id) {
        return getDatabaseService().getQueryable(DatasetProfile.class).where((builder, root) -> builder.equal(root.get("id"), id)).getSingle();
    }

    @Override
    public QueryableList<DatasetProfile> getAll() {
        return getDatabaseService().getQueryable(DatasetProfile.class);
    }

    @Override
    public void delete(DatasetProfile item) {
        this.getDatabaseService().delete(item);
    }

    @Override
    public QueryableList<DatasetProfile> asQueryable() {
        return this.getDatabaseService().getQueryable(DatasetProfile.class);
    }

    @Override
    public CompletableFuture<DatasetProfile> createOrUpdateAsync(DatasetProfile item) {
        return CompletableFuture.supplyAsync(() -> this.createOrUpdate(item));
    }

    @Override
    public DatasetProfile find(UUID id, String hint) {
        throw new UnsupportedOperationException();
    }
}
