import { DataTableRequest } from '../../models/data-table/DataTableRequest';
import { UserErrorModel } from '../../models/users/UserErrorModel';
import { UserReferenceService } from '../../services/user-reference/user-reference-data.service';
import { UserListingModel } from '../../models/users/UserListingModel';
import { SnackBarNotificationComponent } from '../../shared/components/notificaiton/snack-bar-notification.component';
import { UserCriteria } from '../../models/criteria/users/UserCriteria';
import { UserCriteriaErrorModel } from '../../models/criteria/users/UserCriteriaErrorModel';
import { Observable } from 'rxjs/Rx';
import { Component, OnInit, AfterViewInit, ViewChild } from '@angular/core';
import { UsersCriteriaComponent } from '../../shared/components/criteria/users/users-criteria.component';
import { Router } from '@angular/router';
import { Principal } from '../../models/login/Principal';
import { MatPaginator, MatSort, MatSnackBar } from '@angular/material';
import { TranslateService } from '@ngx-translate/core';
import { DataSource } from '@angular/cdk/table';
import { RecentActivityTypes } from '../../users/activity/RecentActivityTypes';


@Component({
	selector: 'app-recent-activity',
	templateUrl: './recent-activity.component.html',
	styleUrls: ['./recent-activity.component.scss'],
	providers: [
		UserReferenceService
	]
})
export class RecentActivityComponent implements OnInit {

	datasetActivities: any[];
	projectActivities: any[];
	dmpActivities: any[];
	public RecentActivityTypes = RecentActivityTypes;
	constructor(
		private router: Router,
		private userReferenceService: UserReferenceService
	) { }

	ngOnInit() {
		this.userReferenceService.getRecentActivity().subscribe(response => {
			this.datasetActivities = response["recentDatasetActivities"];
			this.dmpActivities = response["recentDmpActivities"];
			this.projectActivities = response["recentProjectActivities"];
		})
	}

	redirect(id: string, type: RecentActivityTypes) {
		switch (type) {
			case RecentActivityTypes.PROJECT: {
				this.router.navigate(["projects/edit/" + id]);
				return;
			}
			case RecentActivityTypes.DATASET: {
				this.router.navigate(["datasets/edit/" + id]);
				return;
			}
			case RecentActivityTypes.DMP: {
				this.router.navigate(["dmps/edit/" + id]);
				return;
			}
			default: throw new Error("Unsupported Activity Type ")
		}

	}
}